@@ -14,11 +14,12 @@
 #include <QQuickWindow>
 
 #include <KAboutData>
-#ifndef Q_OS_ANDROID
+#ifdef HAVE_KDBUSADDONS
 #include <KDBusService>
 #endif
 #include <KLocalizedContext>
 #include <KLocalizedString>
+#include <KWindowConfig>
 
 #include "neochat-version.h"
 
@@ -28,9 +29,13 @@
 #include "controller.h"
 #include "csapi/joining.h"
 #include "csapi/leaving.h"
+#include "devicesmodel.h"
 #include "emojimodel.h"
+#include "filetypesingleton.h"
+#include "login.h"
 #include "matriximageprovider.h"
 #include "messageeventmodel.h"
+#include "messagefiltermodel.h"
 #include "neochatconfig.h"
 #include "neochatroom.h"
 #include "neochatuser.h"
@@ -41,6 +46,7 @@
 #include "sortfilterroomlistmodel.h"
 #include "userdirectorylistmodel.h"
 #include "userlistmodel.h"
+#include "actionshandler.h"
 
 using namespace Quotient;
 
@@ -64,6 +70,10 @@ int main(int argc, char *argv[])
     }
 #endif
 
+#ifdef Q_OS_WINDOWS
+    QApplication::setStyle(QStringLiteral("breeze"));
+#endif
+
     QApplication::setOrganizationName("KDE");
 
     KAboutData about(QStringLiteral("neochat"), i18n("Neochat"), QStringLiteral(NEOCHAT_VERSION_STRING), i18n("Matrix client"), KAboutLicense::GPL_V3, i18n("© 2018-2020 Black Hat, 2020 KDE Community"));
@@ -75,32 +85,39 @@ int main(int argc, char *argv[])
     KAboutData::setApplicationData(about);
     QApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("org.kde.neochat")));
 
-#ifndef Q_OS_ANDROID
+#ifdef HAVE_KDBUSADDONS
     KDBusService service(KDBusService::Unique);
 #endif
 
 #ifdef NEOCHAT_FLATPAK
     // Copy over the included FontConfig configuration to the
     // app's config dir:
-    QFile::copy("/app/etc/fonts/conf.d/99-noto-mono-color-emoji.conf",
-                "/var/config/fontconfig/conf.d/99-noto-mono-color-emoji.conf");
+    QFile::copy("/app/etc/fonts/conf.d/99-noto-mono-color-emoji.conf", "/var/config/fontconfig/conf.d/99-noto-mono-color-emoji.conf");
 #endif
 
     Clipboard clipboard;
     auto config = NeoChatConfig::self();
+    FileTypeSingleton fileTypeSingleton;
+
+    Login *login = new Login();
 
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "Controller", &Controller::instance());
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "Clipboard", &clipboard);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "Config", config);
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "FileType", &fileTypeSingleton);
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "LoginHelper", login);
     qmlRegisterType<AccountListModel>("org.kde.neochat", 1, 0, "AccountListModel");
+    qmlRegisterType<ActionsHandler>("org.kde.neochat", 1, 0, "ActionsHandler");
     qmlRegisterType<ChatDocumentHandler>("org.kde.neochat", 1, 0, "ChatDocumentHandler");
     qmlRegisterType<RoomListModel>("org.kde.neochat", 1, 0, "RoomListModel");
     qmlRegisterType<UserListModel>("org.kde.neochat", 1, 0, "UserListModel");
     qmlRegisterType<MessageEventModel>("org.kde.neochat", 1, 0, "MessageEventModel");
+    qmlRegisterType<MessageFilterModel>("org.kde.neochat", 1, 0, "MessageFilterModel");
     qmlRegisterType<PublicRoomListModel>("org.kde.neochat", 1, 0, "PublicRoomListModel");
     qmlRegisterType<UserDirectoryListModel>("org.kde.neochat", 1, 0, "UserDirectoryListModel");
     qmlRegisterType<EmojiModel>("org.kde.neochat", 1, 0, "EmojiModel");
     qmlRegisterType<SortFilterRoomListModel>("org.kde.neochat", 1, 0, "SortFilterRoomListModel");
+    qmlRegisterType<DevicesModel>("org.kde.neochat", 1, 0, "DevicesModel");
     qmlRegisterUncreatableType<RoomMessageEvent>("org.kde.neochat", 1, 0, "RoomMessageEvent", "ENUM");
     qmlRegisterUncreatableType<RoomType>("org.kde.neochat", 1, 0, "RoomType", "ENUM");
     qmlRegisterUncreatableType<UserType>("org.kde.neochat", 1, 0, "UserType", "ENUM");
@@ -114,6 +131,7 @@ int main(int argc, char *argv[])
     qRegisterMetaType<NeoChatRoom *>("NeoChatRoom*");
     qRegisterMetaType<NeoChatUser *>("NeoChatUser*");
     qRegisterMetaType<GetRoomEventsJob *>("GetRoomEventsJob*");
+    qRegisterMetaType<QMimeType>("QMimeType");
 
     qRegisterMetaTypeStreamOperators<Emoji>();
 
@@ -139,7 +157,7 @@ int main(int argc, char *argv[])
         return -1;
     }
 
-#ifndef Q_OS_ANDROID
+#ifdef HAVE_KDBUSADDONS
     QObject::connect(&service, &KDBusService::activateRequested, &engine, [&engine](const QStringList & /*arguments*/, const QString & /*workingDirectory*/) {
         const auto rootObjects = engine.rootObjects();
         for (auto obj : rootObjects) {
@@ -151,6 +169,17 @@ int main(int argc, char *argv[])
             }
         }
     });
+    const auto rootObjects = engine.rootObjects();
+    for (auto obj : rootObjects) {
+        auto view = qobject_cast<QQuickWindow*>(obj);
+        if (view) {
+            KConfig dataResource("data", KConfig::SimpleConfig, QStandardPaths::AppDataLocation);
+            KConfigGroup windowGroup(&dataResource, "Window");
+            KWindowConfig::restoreWindowSize(view, windowGroup);
+            KWindowConfig::restoreWindowPosition(view, windowGroup);
+            break;
+        }
+    }
 #endif
     return QApplication::exec();
 }
