@@ -11,6 +11,7 @@
 #include <events/redactionevent.h>
 #include <events/roomavatarevent.h>
 #include <events/roommemberevent.h>
+#include <events/stickerevent.h>
 #include <events/simplestateevents.h>
 #include <user.h>
 
@@ -45,6 +46,7 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
     roles[ShowSectionRole] = "showSection";
     roles[ReactionRole] = "reaction";
     roles[IsEditedRole] = "isEdited";
+    roles[FormattedBodyRole] = "formattedBody";
     return roles;
 }
 
@@ -62,8 +64,8 @@ MessageEventModel::MessageEventModel(QObject *parent)
             return;
         }
         m_currentRoom->getPreviousContent(50);
-        connect(this, &QAbstractListModel::rowsInserted, this, [=](){
-            if(m_currentRoom->readMarkerEventId().isEmpty()) {
+        connect(this, &QAbstractListModel::rowsInserted, this, [=]() {
+            if (m_currentRoom->readMarkerEventId().isEmpty()) {
                 return;
             }
             const auto it = m_currentRoom->findInTimeline(m_currentRoom->readMarkerEventId());
@@ -90,6 +92,9 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
     m_currentRoom = room;
     if (room) {
         room->setDisplayed();
+        if (m_currentRoom->timelineSize() < 10) {
+            room->getPreviousContent(50);
+        }
         lastReadEventId = room->readMarkerEventId();
 
         using namespace Quotient;
@@ -296,7 +301,7 @@ int MessageEventModel::rowCount(const QModelIndex &parent) const
 inline QVariantMap userAtEvent(NeoChatUser *user, NeoChatRoom *room, const RoomEvent &evt)
 {
     Q_UNUSED(evt)
-    return QVariantMap {
+    return QVariantMap{
         {"isLocalUser", user->id() == room->localUser()->id()},
         {"id", user->id()},
         {"avatarMediaId", user->avatarMediaId(room)},
@@ -326,9 +331,20 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
             auto reason = evt.redactedBecause()->reason();
             return (reason.isEmpty()) ? i18n("<i>[This message was deleted]</i>") : i18n("<i>[This message was deleted: %1]</i>").arg(evt.redactedBecause()->reason());
         }
+
         return m_currentRoom->eventToString(evt, Qt::RichText);
     }
 
+    if (role == FormattedBodyRole) {
+        if (auto e = eventCast<const RoomMessageEvent>(&evt)) {
+            if (e->hasTextContent() && e->mimeType().name() != "text/plain") {
+                return static_cast<const Quotient::EventContent::TextContent *>(e->content())->body;
+            }
+        }
+
+        return {};
+    }
+
     if (role == MessageRole) {
         return m_currentRoom->eventToString(evt);
     }
@@ -359,6 +375,9 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
 
             return "message";
         }
+        if (is<const StickerEvent>(evt)) {
+            return "sticker";
+        }
         if (evt.isStateEvent()) {
             return "state";
         }
@@ -395,6 +414,10 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
             // content JSON stored in EventContent::Base
             return e->hasFileContent() ? QVariant::fromValue(e->content()->originalJson) : QVariant();
         };
+
+        if (auto e = eventCast<const StickerEvent>(&evt)) {
+            return QVariant::fromValue(e->image().originalJson);
+        }
     }
 
     if (role == HighlightRole) {
@@ -463,6 +486,9 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
                 return QVariant::fromValue(m_currentRoom->fileTransferInfo(e->id()));
             }
         }
+        if (auto e = eventCast<const StickerEvent>(&evt)) {
+            return QVariant::fromValue(m_currentRoom->fileTransferInfo(e->id()));
+        }
     }
 
     if (role == AnnotationRole) {
@@ -499,7 +525,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         };
         const auto &replyEvt = **replyIt;
 
-        return QVariantMap {{"eventId", replyEventId}, {"display", m_currentRoom->eventToString(replyEvt, Qt::RichText)}, {"author", userAtEvent(static_cast<NeoChatUser *>(m_currentRoom->user(replyEvt.senderId())), m_currentRoom, evt)}};
+        return QVariantMap{{"eventId", replyEventId}, {"display", m_currentRoom->eventToString(replyEvt, Qt::RichText)}, {"author", userAtEvent(static_cast<NeoChatUser *>(m_currentRoom->user(replyEvt.senderId())), m_currentRoom, evt)}};
     }
 
     if (role == ShowAuthorRole) {
@@ -553,7 +579,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
                 authors.append(userAtEvent(author, m_currentRoom, evt));
             }
             bool hasLocalUser = i.value().contains(static_cast<NeoChatUser *>(m_currentRoom->localUser()));
-            res.append(QVariantMap {{"reaction", i.key()}, {"count", i.value().count()}, {"authors", authors}, {"hasLocalUser", hasLocalUser}});
+            res.append(QVariantMap{{"reaction", i.key()}, {"count", i.value().count()}, {"authors", authors}, {"hasLocalUser", hasLocalUser}});
             ++i;
         }
 
