@@ -27,12 +27,13 @@
 #include "events/roomcanonicalaliasevent.h"
 #include "events/roommessageevent.h"
 #include "events/roompowerlevelsevent.h"
+#include "events/stickerevent.h"
 #include "events/typingevent.h"
 #include "jobs/downloadfilejob.h"
+#include "neochatconfig.h"
 #include "notificationsmanager.h"
 #include "user.h"
 #include "utils.h"
-#include "neochatconfig.h"
 
 #include <KLocalizedString>
 
@@ -50,7 +51,7 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
             return;
         }
         const RoomEvent *lastEvent = messageEvents().rbegin()->get();
-        if(lastEvent->originTimestamp() < QDateTime::currentDateTime().addSecs(-60)) {
+        if (lastEvent->originTimestamp() < QDateTime::currentDateTime().addSecs(-60)) {
             return;
         }
         if (lastEvent->isStateEvent()) {
@@ -61,14 +62,25 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
             return;
         }
 
-        NotificationsManager::instance().postNotification(this, lastEvent->id(), displayName(), sender->displayname(this), eventToString(*lastEvent), avatar(128));
+        QImage avatar_image;
+        if (!sender->avatarUrl(this).isEmpty()) {
+            avatar_image = sender->avatar(128, this);
+        } else {
+            avatar_image = this->avatar(128);
+        }
+
+        NotificationsManager::instance().postNotification(this, displayName(), sender->displayname(this), eventToString(*lastEvent), avatar_image);
     });
 
-    connect(this, &Room::aboutToAddHistoricalMessages,
-            this, &NeoChatRoom::readMarkerLoadedChanged);
+    connect(this, &Room::aboutToAddHistoricalMessages, this, &NeoChatRoom::readMarkerLoadedChanged);
 
-    connect(this, &Quotient::Room::eventsHistoryJobChanged,
-            this, &NeoChatRoom::lastActiveTimeChanged);
+    connect(this, &Quotient::Room::eventsHistoryJobChanged, this, &NeoChatRoom::lastActiveTimeChanged);
+
+    connect(this, &Room::joinStateChanged, this, [=](JoinState oldState, JoinState newState) {
+        if(oldState == JoinState::Invite && newState != JoinState::Invite) {
+            Q_EMIT isInviteChanged();
+        }
+    });
 }
 
 void NeoChatRoom::uploadFile(const QUrl &url, const QString &body)
@@ -93,7 +105,6 @@ void NeoChatRoom::uploadFile(const QUrl &url, const QString &body)
     });
     connect(this, &Room::fileTransferProgress, [=](const QString &id, qint64 progress, qint64 total) {
         if (id == txnId) {
-            qDebug() << "Progress:" << progress << total;
             setFileUploadingProgress(int(float(progress) / float(total) * 100));
         }
     });
@@ -115,7 +126,12 @@ QVariantList NeoChatRoom::getUsersTyping() const
     users.removeAll(localUser());
     QVariantList userVariants;
     for (User *user : users) {
-        userVariants.append(QVariant::fromValue(user));
+        userVariants.append(QVariantMap {
+            {"id", user->id()},
+            {"avatarMediaId", user->avatarMediaId(this)},
+            {"displayName", user->displayname(this)},
+            {"display", user->name()},
+        });
     }
     return userVariants;
 }
@@ -161,12 +177,11 @@ const RoomMessageEvent *NeoChatRoom::lastEvent(bool ignoreStateEvent) const
 QString NeoChatRoom::lastEventToString() const
 {
     if (auto event = lastEvent()) {
-        return user(event->senderId())->displayname() + (event->isStateEvent() ? " " : ": ") + eventToString(*event);
+        return user(event->senderId())->displayname(this) + (event->isStateEvent() ? " " : ": ") + eventToString(*event);
     }
     return QLatin1String("");
 }
 
-
 bool NeoChatRoom::isEventHighlighted(const RoomEvent *e) const
 {
     return highlights.contains(e);
@@ -261,7 +276,15 @@ QVariantList NeoChatRoom::getUsers(const QString &keyword) const
     QVariantList matchedList;
     for (const auto u : userList) {
         if (u->displayname(this).contains(keyword, Qt::CaseInsensitive)) {
-            matchedList.append(QVariant::fromValue(u));
+            NeoChatUser user(u->id(), u->connection());
+            QVariantMap userVariant {
+                { QStringLiteral("id"), user.id() },
+                { QStringLiteral("displayName"), user.displayname(this) },
+                { QStringLiteral("avatarMediaId"), user.avatarMediaId(this) },
+                { QStringLiteral("color"), user.color() }
+            };
+
+            matchedList.append(QVariant::fromValue(userVariant));
         }
     }
 
@@ -341,6 +364,9 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
             }
             return plainBody;
         },
+        [](const StickerEvent &e) {
+            return e.body();
+        },
         [this](const RoomMemberEvent &e) {
             // FIXME: Rewind to the name that was at the time of this event
             auto subjectName = this->user(e.userId())->displayname();
@@ -348,25 +374,38 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
             switch (e.membership()) {
             case MembershipType::Invite:
                 if (e.repeatsState()) {
-                    return i18n("reinvited %1 to the room", subjectName);
+                    auto text = i18n("reinvited %1 to the room", subjectName);
+                    if (!e.reason().isEmpty()) {
+                        text += i18nc("Optional reason for an invitation", ": %1") + e.reason().toHtmlEscaped();
+                    }
+                    return text;
                 }
-                break;
+                Q_FALLTHROUGH();
             case MembershipType::Join: {
-                if (e.repeatsState()) {
-                    return i18n("joined the room (repeated)");
-                }
-                if (!e.prevContent() || e.membership() != e.prevContent()->membership) {
-                    return e.membership() == MembershipType::Invite ? i18n("invited %1 to the room", subjectName) : i18n("joined the room");
-                }
                 QString text {};
-                if (e.isRename()) {
-                    if (e.displayName().isEmpty()) {
+                // Part 1: invites and joins
+                if (e.repeatsState()) {
+                    text = i18n("joined the room (repeated)");
+                } else if (e.changesMembership()) {
+                    text = e.membership() == MembershipType::Invite
+                            ? i18n("invited %1 to the room", subjectName)
+                            : i18n("joined the room");
+                }
+                if (!text.isEmpty()) {
+                    if (!e.reason().isEmpty()) {
+                        text += i18n(": %1", e.reason().toHtmlEscaped());
+                    }
+                    return text;
+                }
+                // Part 2: profile changes of joined members
+                if (e.isRename() && NeoChatConfig::self()->showRename()) {
+                    if (!e.displayName().isEmpty()) {
                         text = i18n("cleared their display name");
                     } else {
                         text = i18n("changed their display name to %1", e.displayName().toHtmlEscaped());
                     }
                 }
-                if (e.isAvatarUpdate()) {
+                if (e.isAvatarUpdate() && NeoChatConfig::self()->showAvatarUpdate()) {
                     if (!text.isEmpty()) {
                         text += i18n(" and ");
                     }
@@ -427,7 +466,7 @@ void NeoChatRoom::changeAvatar(const QUrl &localFile)
     const auto job = connection()->uploadFile(localFile.toLocalFile());
     if (isJobRunning(job)) {
         connect(job, &BaseJob::success, this, [this, job] {
-            connection()->callApi<SetRoomStateWithKeyJob>(id(), "m.room.avatar", localUser()->id(), QJsonObject {{"url", job->contentUri()}});
+            connection()->callApi<SetRoomStateWithKeyJob>(id(), "m.room.avatar", localUser()->id(), QJsonObject{{"url", job->contentUri()}});
         });
     }
 }
@@ -475,18 +514,6 @@ QString NeoChatRoom::markdownToHTML(const QString &markdown)
     return result;
 }
 
-void NeoChatRoom::postArbitaryMessage(const QString &text, Quotient::RoomMessageEvent::MsgType type, const QString &replyEventId)
-{
-    const auto parsedHTML = markdownToHTML(text);
-    const bool isRichText = Qt::mightBeRichText(parsedHTML);
-
-    if (isRichText) { // Markdown
-        postHtmlMessage(text, parsedHTML, type, replyEventId);
-    } else { // Plain text
-        postPlainMessage(text, type, replyEventId);
-    }
-}
-
 QString msgTypeToString(MessageEventType msgType)
 {
     switch (msgType) {
@@ -511,59 +538,52 @@ QString msgTypeToString(MessageEventType msgType)
     }
 }
 
-void NeoChatRoom::postPlainMessage(const QString &text, MessageEventType type, const QString &replyEventId)
+void NeoChatRoom::postMessage(const QString &rawText, const QString &text, MessageEventType type, const QString &replyEventId, const QString &relateToEventId)
 {
-    bool isReply = !replyEventId.isEmpty();
-    const auto replyIt = findInTimeline(replyEventId);
-    if (replyIt == timelineEdge()) {
-        isReply = false;
-    }
-
-    if (isReply) {
-        const auto &replyEvt = **replyIt;
-
-        // clang-format off
-        QJsonObject json{
-          {"msgtype", msgTypeToString(type)},
-          {"body", "> <" + replyEvt.senderId() + "> " + eventToString(replyEvt) + "\n\n" + text},
-          {"format", "org.matrix.custom.html"},
-          {"m.relates_to",
-            QJsonObject {
-              {"m.in_reply_to",
-                QJsonObject {
-                  {"event_id", replyEventId}
-                }
-              }
-            }
-          },
-          {"formatted_body",
-            "<mx-reply><blockquote><a href=\"https://matrix.to/#/" +
-            id() + "/" +
-            replyEventId +
-            "\">In reply to</a> <a href=\"https://matrix.to/#/" +
-            replyEvt.senderId() + "\">" + replyEvt.senderId() +
-            "</a><br>" + eventToString(replyEvt, Qt::RichText) +
-            "</blockquote></mx-reply>" + text
-          }
-        };
-        // clang-format on
-
-        postJson("m.room.message", json);
-
-        return;
-    }
-
-    Room::postMessage(text, type);
+    const auto html = markdownToHTML(text);
+    QString cleanText(text);
+    cleanText.replace(QRegularExpression("\\[(.+)\\]\\(.+\\)"), "\\1");
+    postHtmlMessage(rawText, html, type, replyEventId, relateToEventId);
 }
 
-void NeoChatRoom::postHtmlMessage(const QString &text, const QString &html, MessageEventType type, const QString &replyEventId)
+void NeoChatRoom::postHtmlMessage(const QString &text, const QString &html, MessageEventType type, const QString &replyEventId, const QString &relateToEventId)
 {
+    bool isRichText = Qt::mightBeRichText(html);
     bool isReply = !replyEventId.isEmpty();
+    bool isEdit = !relateToEventId.isEmpty();
     const auto replyIt = findInTimeline(replyEventId);
     if (replyIt == timelineEdge()) {
         isReply = false;
     }
 
+
+    if (isEdit) {
+        QJsonObject json {
+            {"type", "m.room.message"},
+            {"msgtype", msgTypeToString(type)},
+            {"body", "* " + text},
+            {"format", "org.matrix.custom.html"},
+            {"formatted_body", html},
+            {"m.new_content",
+              QJsonObject {
+                {"body", text},
+                {"msgtype", msgTypeToString(type)},
+                {"format", "org.matrix.custom.html"},
+                {"formatted_body", html}
+              }
+            },
+            {"m.relates_to",
+                QJsonObject {
+                    {"rel_type", "m.replace"},
+                    {"event_id", relateToEventId}
+                }
+            }
+        };
+
+        postJson("m.room.message", json);
+        return;
+    }
+
     if (isReply) {
         const auto &replyEvt = **replyIt;
 
@@ -588,7 +608,7 @@ void NeoChatRoom::postHtmlMessage(const QString &text, const QString &html, Mess
             "\">In reply to</a> <a href=\"https://matrix.to/#/" +
             replyEvt.senderId() + "\">" + replyEvt.senderId() +
             "</a><br>" + eventToString(replyEvt, Qt::RichText) +
-            "</blockquote></mx-reply>" + html
+            "</blockquote></mx-reply>" + (isRichText ? html : text)
           }
         };
         // clang-format on
@@ -598,7 +618,11 @@ void NeoChatRoom::postHtmlMessage(const QString &text, const QString &html, Mess
         return;
     }
 
-    Room::postHtmlMessage(text, html, type);
+    if (isRichText) {
+        Room::postHtmlMessage(text, html, type);
+    } else {
+        Room::postMessage(text, type);
+    }
 }
 
 void NeoChatRoom::toggleReaction(const QString &eventId, const QString &reaction)
@@ -675,3 +699,8 @@ bool NeoChatRoom::readMarkerLoaded() const
     const auto it = findInTimeline(readMarkerEventId());
     return it != timelineEdge();
 }
+
+bool NeoChatRoom::isInvite() const
+{
+    return joinState() == JoinState::Invite;
+}
