@@ -3,6 +3,7 @@
  *
  * SPDX-License-Identifier: GPL-3.0-only
  */
+
 #pragma once
 
 #include <events/encryptionevent.h>
@@ -32,6 +33,7 @@ class NeoChatRoom : public Room
     Q_PROPERTY(QString avatarMediaId READ avatarMediaId NOTIFY avatarChanged STORED false)
     Q_PROPERTY(bool readMarkerLoaded READ readMarkerLoaded NOTIFY readMarkerLoadedChanged)
     Q_PROPERTY(QDateTime lastActiveTime READ lastActiveTime NOTIFY lastActiveTimeChanged)
+    Q_PROPERTY(bool isInvite READ isInvite NOTIFY isInviteChanged)
 
 public:
     explicit NeoChatRoom(Connection *connection, QString roomId, JoinState joinState = {});
@@ -102,6 +104,8 @@ public:
     Q_INVOKABLE [[nodiscard]] bool canSendEvent(const QString &eventType) const;
     Q_INVOKABLE [[nodiscard]] bool canSendState(const QString &eventType) const;
 
+    bool isInvite() const;
+
 private:
     QString m_cachedInput;
     QSet<const Quotient::RoomEvent *> highlights;
@@ -128,15 +132,17 @@ Q_SIGNALS:
     void backgroundChanged();
     void readMarkerLoadedChanged();
     void lastActiveTimeChanged();
+    void isInviteChanged();
 
 public Q_SLOTS:
-    void uploadFile(const QUrl &url, const QString &body = "");
+    void uploadFile(const QUrl &url, const QString &body = QString());
     void acceptInvitation();
     void forget();
     void sendTypingNotification(bool isTyping);
-    void postArbitaryMessage(const QString &text, Quotient::RoomMessageEvent::MsgType type, const QString &replyEventId);
-    void postPlainMessage(const QString &text, Quotient::RoomMessageEvent::MsgType type = Quotient::MessageEventType::Text, const QString &replyEventId = "");
-    void postHtmlMessage(const QString &text, const QString &html, Quotient::MessageEventType type = Quotient::MessageEventType::Text, const QString &replyEventId = "");
+    /// @param rawText The text as it was typed.
+    /// @param cleanedText The text with link to the users.
+    void postMessage(const QString &rawText, const QString &cleanedText, Quotient::MessageEventType type = Quotient::MessageEventType::Text, const QString &replyEventId = QString(), const QString &relateToEventId = QString());
+    void postHtmlMessage(const QString &text, const QString &html, Quotient::MessageEventType type = Quotient::MessageEventType::Text, const QString &replyEventId = QString(), const QString &relateToEventId = QString());
     void changeAvatar(const QUrl &localFile);
     void addLocalAlias(const QString &alias);
     void removeLocalAlias(const QString &alias);
